@@ -8,17 +8,40 @@ from ..symbol import SymbolType
 from ...address_translator import AT
 from ...utils import stream_or_path
 from elftools.elf.descriptions import describe_ei_osabi
+from elftools.elf.dynamic import DynamicSection
+from elftools.elf.enums import ENUM_DT_FLAGS
+from enum import Enum
 from collections import OrderedDict
 
 __all__ = ('MetaELF',)
 
 l = logging.getLogger(name=__name__)
 
+class Relro(Enum):
+    NONE = 0
+    PARTIAL = 1
+    FULL = 2
+
 def maybedecode(string):
     # so... it turns out that pyelftools is garbage and will transparently give you either strings or bytestrings
     # based on pretty much nothing whatsoever
     return string if type(string) is str else string.decode()
 
+def get_relro(elf):
+    # The tests for partial and full RELRO have been taken from
+    # checksec.sh v1.5 (https://www.trapkit.de/tools/checksec/):
+    #   - Partial RELRO has a 'GNU_RELRO' segment
+    #   - Full RELRO also has a 'BIND_NOW' flag in the dynamic section
+    if not any(seg.header.p_type == 'PT_GNU_RELRO' for seg in elf.iter_segments()):
+        return Relro.NONE
+    dyn_sec = elf.get_section_by_name('.dynamic')
+    if dyn_sec is None or not isinstance(dyn_sec, DynamicSection):
+        return Relro.PARTIAL
+    flags = [tag for tag in dyn_sec.iter_tags() if tag.entry.d_tag == 'DT_FLAGS']
+    if len(flags) != 1:
+        return Relro.PARTIAL
+    return Relro.FULL if flags[0].entry.d_val & ENUM_DT_FLAGS['DF_BIND_NOW'] == ENUM_DT_FLAGS['DF_BIND_NOW'] else Relro.PARTIAL
+
 class MetaELF(Backend):
     """
     A base class that implements functions used by all backends that can load an ELF.
@@ -28,6 +51,7 @@ class MetaELF(Backend):
         tmp_reader = elftools.elf.elffile.ELFFile(self._binary_stream)
         self.os = describe_ei_osabi(tmp_reader.header.e_ident.EI_OSABI)
         self.elfflags = tmp_reader.header.e_flags
+        self.relro = get_relro(tmp_reader)
         self._plt = {}
         self._ppc64_abiv1_initial_rtoc = None
 
@@ -53,9 +77,12 @@ class MetaELF(Backend):
         if self.arch.name != 'X86':
             return False
         # search for tX = GET(ebx) -> Add32(tX, got_addr - addr)
-        if '.got.plt' not in self.sections_map:
+        if '.got.plt' in self.sections_map:
+            got_sec = self.sections_map['.got.plt']
+        elif self.relro is Relro.FULL:
+            got_sec = self.sections_map['.got']
+        else:
             return False
-        got_sec = self.sections_map['.got.plt']
         tx = None
         for stmt in block.statements:
             if stmt.tag == 'Ist_WrTmp' and stmt.data.tag == 'Iex_Get' and stmt.data.offset == self.arch.registers['ebx'][0]:
@@ -278,9 +305,15 @@ class MetaELF(Backend):
                 # WAHP WAHP
                 return
 
+        # some binaries have a bunch of CET stubs before the PLTs, and
+        # in the worst case we might have to skip over each one of
+        # these... so we set the bailout timer accordingly
+        def initial_bailout_timer(func_jmprel):
+            return len(func_jmprel) + 5
+
         if plt_secs:
             # LAST TRY: Find the first block to references ANY GOT slot
-            tick.bailout_timer = 5
+            tick.bailout_timer = initial_bailout_timer (func_jmprel)
             scan_forward(min(plt_sec.vaddr for plt_sec in plt_secs), list(func_jmprel.keys()), push=True)
 
         if not self._plt:
@@ -296,7 +329,7 @@ class MetaELF(Backend):
         name, addr = plt_hitlist[0]
         if addr is None and plt_secs:
             # try to resolve the very first entry
-            tick.bailout_timer = 5
+            tick.bailout_timer = initial_bailout_timer (func_jmprel)
             guessed_addr = min(plt_sec.vaddr for plt_sec in plt_secs)
             scan_forward(guessed_addr, name, push=True)
             if name in self._plt:
